#!/bin/bash -e
#
# freebox-tools:bin/freebox-lcd.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/api/freebox-bash-api.sh" || exit $?

declare -r FREEBOX_BIN="${PROJECT_HOME}/bin/freebox.sh"

function usage {
  cat <<EOF
Assistant pour votre Freebox permettant de gérer le WIFI.

Usage:
  $( basename "${BASH_SOURCE[0]}" ) --help
  $( basename "${BASH_SOURCE[0]}" ) [--debug] --show
  $( basename "${BASH_SOURCE[0]}" ) [--debug] --set [--brightness VALUE] [--orientation VALUE] [--forced-orientation VALUE] [--hide-wifi-key VALUE]
  $( basename "${BASH_SOURCE[0]}" ) [--debug] --reset
  $( basename "${BASH_SOURCE[0]}" ) [--debug] --fun

Actions:
  -h,   --help          Affiche ce message d’aide et quitte.
  -s,   --show          Affiche la configuration actuelle de l'écran LCD
  -R,   --reset         Réinitialise avec les valeurs par défaut
  -S,   --set           Permet de définir des valeurs personnalisées, au moins une valeur
                        est requise.
  -F,   --fun           On s'amuse avec l'écran LCD de la box.

Options:
  -D,   --debug         Affiche les traces

  -k,   --insecure      Autorise les connexions non certifiées lors de l’utilisation de SSL

        --http,
        --allow-http
                        Autorise l’utilisation du protocole http (non sécurisé)

  -B VALUE, --brightness VALUE
                        Lorsque l'action --set est définie, permet de modifier l'intensité
                        de l'affichage de la box. La valeur doit être un entier de 0 à 100.

  -O VALUE, --orientation VALUE
                        Lorsque l'action --set est définie, integer: 0|90|180|270

  -K VALUE, --forced-orientation VALUE
                        Lorsque l'action --set est définie, boolean: true|false

  -W VALUE, --hide-wifi-key VALUE
                        Lorsque l'action --set est définie, boolean: true|false

EOF
}

function run_freebox {
  local result=
  result="$( "${FREEBOX_BIN}" "$@" )" || return $?

  response_check_success "${result}" || return $?
  jq '.result' <<<"${result}" || return $?
}

function get_lcd_config {
  # GET /api/v8/lcd/config/ HTTP/1.1
  run_freebox GET '/lcd/config/' || return $?
}

function _set_lcd_config_ {
  local -r json="$1"

  # PUT /api/v8/lcd/config/ HTTP/1.1
  # { "brightness": 50, "orientation": 0, "orientation_forced": false "hide_wifi_key": false }
  run_freebox PUT '/lcd/config/' --data "${json}" || return $?
}

function set_lcd_config {
  local -r brightness="$1" # integer
  local -r orientation="$2" # integer
  local -r orientation_forced="$3" # boolean
  local -r hide_wifi_key="$4" # boolean

  local data='{}'
  if [ -n "${brightness}" ] ; then
    data="$( jq -c --arg int "${brightness}" '. += { brightness: ( $int | tonumber ) }' <<<"${data}" )" || return $?
  fi
  if [ -n "${orientation}" ] ; then
    data="$( jq -c --arg int "${orientation}" '. += { orientation: ( $int | tonumber ) }' <<<"${data}" )" || return $?
  fi
  if [ -n "${orientation_forced}" ] ; then
    data="$( jq -c --arg bool "${orientation_forced}" '. += { orientation_forced: ( $bool | test("true") ) }' <<<"${data}" )" || return $?
  fi
  if [ -n "${hide_wifi_key}" ] ; then
    data="$( jq -c --arg bool "${hide_wifi_key}" '. += { hide_wifi_key: ( $bool | test("true") ) }' <<<"${data}" )" || return $?
  fi
  _set_lcd_config_ "${data}" || return $?
}

function set_default_lcd_config {
  _set_lcd_config_ '{"orientation":0,"brightness":100,"hide_wifi_key":false,"orientation_forced":false}' || return $?
}

function lcd_config_fun {
  local brightness=50 # integer
  local -i orientation=90 # integer
  local orientation_forced='true' # boolean
  local hide_wifi_key= # boolean (not set)

  trap set_default_lcd_config EXIT

  local -i i=0
  for i in {1..100} ; do
    orientation=$(( 90*(i%4) ))
    brightness=$(( (i*31)%75 + 25 ))
    set_lcd_config "${brightness}" "${orientation}" "${orientation_forced}" "${hide_wifi_key}" | jq -c -s '.' || return $?
    sleep 1
  done
}

function main {
  local param=
  local param_lcd_action=
  local param_set_brightness=           # integer: [0-100]
  local param_set_orientation=          # integer: 0|90|180|270
  local param_set_orientation_forced=   # boolean: true|false
  local param_set_hide_wifi_key=        # boolean: true|false

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage 'true'
        return 0
        ;;

      '-s'|'--show')
        param_lcd_action='show'
        ;;

      '-R'|'--reset')
        param_lcd_action='reset'
        ;;

      '-F'|'--fun')
        param_lcd_action='fun-lcd'
        ;;

      '-S'|'--set')
        param_lcd_action='set'
        ;;

      '-B'|'--brightness') # Imply '--set'
        param_set_brightness="$( P_param "$@" )" && shift || return $? # integer: [0-100]
        ;;

      '-O'|'--orientation') # Imply '--set'
        param_set_orientation="$( P_param "$@" )" && shift || return $? # integer: 0|90|180|270
        ;;

      '-K'|'--forced-orientation') # Imply '--set'
        param_set_orientation_forced="$( P_param "$@" )" && shift || return $? # boolean: true|false
        ;;

      '-W'|'--hide-wifi-key') # Imply '--set'
        param_set_hide_wifi_key="$( P_param "$@" )" && shift || return $? # boolean: true|false
        ;;

      '-k'|'--insecure')
        declare -gx FREEBOX_INSECURE='true'
        ;;

      '--http'|'--allow-http')
        declare -gx FREEBOX_HTTPS_ONLY='true'
        ;;

      '-D'|'--debug')
        declare -gx FREEBOX_DEBUG_CURL='true'
        declare -gx FREEBOX_DEBUG_CURL_OUTPUT='true'
        ;;

      *)
        # unknown option
        echofatal "Paramètre inconnu: '${param}'."
        usage >&2
        return 1
        ;;
    esac
  done

  if [ -n "${param_set_brightness}" ] ; then # integer: [0-100]
    if ! is_positive_integer "${param_set_brightness}" ; then
      echofatal "--brightness VALUE : VALUE must be a positive integer"
      return 1
    fi
    if [ "${param_set_brightness}" -gt 100 ] ; then
      echofatal "--brightness VALUE : VALUE must be a positive integer less or egual to 100"
      return 1
    fi
  fi
  if [ -n "${param_set_orientation}" ] ; then # integer: 0|90|180|270
    case "${param_set_orientation}" in
      '0'|'90'|'180'|'270') ;; # OK
      *)
        echofatal "--orientation VALUE : VALUE must be 0, 90, 180 or 270"
        return 1
        ;;
    esac
  fi
  if [ -n "${param_set_orientation_forced}" ] ; then # boolean: true|false
    if ! is_boolean "${param_set_orientation_forced}" ; then
      echofatal "---forced-orientation VALUE : VALUE must be true or false"
      return 1
    fi
  fi
  if [ -n "${param_set_hide_wifi_key}" ] ; then # boolean: true|false
    if ! is_boolean "${param_set_hide_wifi_key}" ; then
      echofatal "--hide-wifi-key VALUE : VALUE must be true or false"
      return 1
    fi
  fi

  case "${param_lcd_action}" in
    'reset')
      set_default_lcd_config || return $?
      ;;

    'fun-lcd')
      lcd_config_fun || return $?
      ;;

    'show')
      get_lcd_config || return $?
      ;;

    'set')
      if [ -z "${param_set_brightness}${param_set_orientation}${param_set_orientation_forced}${param_set_hide_wifi_key}" ] ; then
        echofatal "--set required at least one of: --brightness, --orientation, ---forced-orientation or --hide-wifi-key"
        return 1
      fi
      set_lcd_config "${param_set_brightness}" "${param_set_orientation}" "${param_set_orientation_forced}" "${param_set_hide_wifi_key}" || return $?
      ;;

    '')
      echofatal 'Action manquante'
      usage >&2
      return 1
      ;;
    *)
      echofatal "Action inconnue: '${param_lcd_action}'"
      usage >&2
      return 1
      ;;
  esac
}

main "$@" || exit $?
